__version__ = "2208.0.1"
AVAILABLE_PYTHON_VERSIONS = ["3.7", "3.8", "3.9", "3.10"]
AVAILABLE_CONDA_ARCHS = [
    "Linux-x86_64",
    "Linux-aarch64",
    "Linux-ppc64le",
    "Linux-s390x",
    "MacOSX-x86_64",
]
